import { deepMerge } from '@empathyco/x-deep-merge';
import { fetchHttpClient } from '../http-clients/fetch.http-client';
import { identityMapper } from '../mappers/identity.mapper';
import { interpolate } from '../utils/interpolate';
/**
 * Factory to create {@link ExtendableEndpointAdapter | endpoint adapters} with the given
 * {@link EndpointAdapterOptions | options}.
 *
 * @param options - The {@link EndpointAdapterOptions | options} to create a new
 * {@link ExtendableEndpointAdapter} with.
 *
 * @returns A brand new {@link ExtendableEndpointAdapter} object.
 * @public
 */
export const endpointAdapterFactory = (options) => {
    const endpointAdapter = (request, { endpoint: requestEndpoint, ...requestOptions } = {}) => {
        const { endpoint: rawEndpoint, httpClient = fetchHttpClient, requestMapper = identityMapper, responseMapper = identityMapper, defaultRequestOptions = {} } = options;
        const endpoint = getEndpoint(requestEndpoint !== null && requestEndpoint !== void 0 ? requestEndpoint : rawEndpoint, request);
        const requestParameters = requestMapper(request, { endpoint });
        return httpClient(endpoint, deepMerge({}, defaultRequestOptions, requestOptions, { parameters: requestParameters })).then(response => responseMapper(response, { endpoint, requestParameters }));
    };
    endpointAdapter.extends = (extendedOptions) => endpointAdapterFactory({
        ...options,
        ...extendedOptions
    });
    return endpointAdapter;
};
/**
 * Returns an endpoint.
 *
 * @param endpoint - The endpoint to process.
 * @param request - The request object.
 *
 * @returns The endpoint.
 * @internal
 */
function getEndpoint(endpoint, request) {
    if (!endpoint) {
        throw Error('Tried to make a request without an endpoint');
    }
    return typeof endpoint === 'function'
        ? endpoint(request, {})
        : interpolate(endpoint, request);
}
//# sourceMappingURL=endpoint-adapter.factory.js.map