import { deepMerge } from '@empathyco/x-deep-merge';
import { isArray, isFunction, isObject, isPath, reduce } from '@empathyco/x-utils';
import { createMutableSchema, isInternalMethod } from '../schemas/utils';
import { extractValue } from '../utils/extract-value';
/**
 * The 'schemaMapperFactory' function creates a {@link Mapper | mapper function} for a given
 * {@link Schema | schema}.
 *
 * @param schema - The {@link Schema | schema} to apply in the {@link Mapper | mapper function}.
 * @returns A {@link Mapper | mapper function} that applies the given {@link Schema | schema}.
 * @public
 */
export function schemaMapperFactory(schema) {
    return function mapper(source, context) {
        return mapSchema(source, schema, context);
    };
}
/**
 * The `mapSchema()` function creates a new object populated with the transformations defined by a
 * {@link Schema} applied to a source object.
 *
 * @param source - The object to apply the transformations to.
 * @param schema - The object that defines the transformations to apply.
 * @param context - The {@link MapperContext | mapper context} to feed the transformations with.
 * @returns A new object with each element being the result of the applied transformation.
 * @internal
 */
function mapSchema(source, schema, context) {
    if (!source) {
        //eslint-disable-next-line no-console
        console.warn('This schema cannot be applied', createMutableSchema(schema));
        return undefined;
    }
    return reduce(schema, (target, key, transformer) => {
        if (typeof transformer === 'string' && isPath(source, transformer)) {
            target[key] = extractValue(source, transformer);
        }
        else if (isFunction(transformer) && !isInternalMethod(transformer.name)) {
            target[key] = transformer(source, context);
        }
        else if (isObject(transformer)) {
            const value = '$subSchema' in transformer
                ? applySubSchemaTransformer(source, transformer, context, schema)
                : mapSchema(source, transformer, context);
            if (value) {
                target[key] = value;
            }
        }
        return target;
    }, {});
}
/**
 * The `applySubSchemaTransformer()` function executes a `mapSchema()` function applying the defined
 * {@link SubSchemaTransformer.$subSchema}.
 *
 * @param source - The object to feed the schema.
 * @param subSchemaTransformer - The {@link SubSchemaTransformer} object with a $path, $subSchema
 * and $context options.
 * @param subSchemaTransformer.$path
 * @param subSchemaTransformer.$subSchema
 * @param rawContext - The {@link MapperContext | mapper context} to feed the mapSchema function.
 * @param subSchemaTransformer.$context
 * @param schema - The {@link Schema} to apply.
 * @returns The result of calling `mapSchema()` with the source, schema and context arguments.
 * @internal
 */
function applySubSchemaTransformer(source, { $subSchema, $path, $context }, rawContext, schema) {
    const subSource = extractValue(source, $path);
    if (!subSource) {
        return;
    }
    const extendedContext = {};
    if ($context) {
        Object.entries($context).forEach(([key, value]) => {
            if (['requestParameters', 'endpoint', 'mappedValue'].includes(key)) {
                return;
            }
            extendedContext[key] = isFunction(value)
                ? value(source)
                : extractValue(source, value);
        });
    }
    const context = deepMerge({}, rawContext, $context, extendedContext);
    let subSchema;
    if ($subSchema === '$self') {
        subSchema = schema;
    }
    else if (isFunction($subSchema)) {
        subSchema = $subSchema(source);
    }
    else {
        subSchema = $subSchema;
    }
    return isArray(subSource)
        ? subSource.map(item => mapSchema(item, subSchema, context))
        : mapSchema(subSource, subSchema, context);
}
//# sourceMappingURL=schema-mapper.factory.js.map